Exception: AMQP::IncompatibleOptionsError

Inherits:
Error
  • Object
show all
Defined in:
lib/amqp/exceptions.rb

Overview

Raised when queue (or exchange) declaration fails because another queue with the same name but different attributes already exists in the channel object cache.

API:

  • public

Instance Attribute Summary

Attributes inherited from Error

#cause

Instance Method Summary collapse

Constructor Details

#initialize(name, opts_1, opts_2) ⇒ IncompatibleOptionsError

Returns a new instance of IncompatibleOptionsError.

API:

  • public



65
66
67
# File 'lib/amqp/exceptions.rb', line 65

def initialize(name, opts_1, opts_2)
  super("There is already an instance called #{name} with options #{opts_1.inspect}, you can't define the same instance with different options (#{opts_2.inspect})!")
end